﻿"use strict";

const chalk = require("chalk");

const info = exports.info = chalk.bold.cyan;
const danger = exports.danger = chalk.bold.red;
const succeed = exports.danger = chalk.bold.green;
const warning = exports.warning = chalk.bold.yellow;

const logInfo = exports.logInfo = (...tokens) => log(info, ...tokens);
const logDanger = exports.logDanger = (...tokens) => log(danger, ...tokens);
const logSucceed = exports.logSucceed = (...tokens) => log(succeed, ...tokens);
const logWarning = exports.logWarning = (...tokens) => log(warning, ...tokens);

const logInfoTitle = exports.logInfoTitle = (title, ...tokens) => logTitle(info, title, ...tokens);
const logDangerTitle = exports.logDangerTitle = (title, ...tokens) => logTitle(danger, title, ...tokens);
const logSucceedTitle = exports.logSucceedTitle = (title, ...tokens) => logTitle(succeed, title, ...tokens);
const logWarningTitle = exports.logWarningTitle = (title, ...tokens) => logTitle(warning, title, ...tokens);

const createInfoTitleLogger = exports.createInfoTitleLogger = title => logInfoTitle.bind(null, title);
const createDangerTitleLogger = exports.createDangerTitleLogger = title => logDangerTitle.bind(null, title);
const createSucceedTitleLogger = exports.createSucceedTitleLogger = title => logSucceedTitle.bind(null, title);
const createWarningTitleLogger = exports.createWarningTitleLogger = title => logWarningTitle.bind(null, title);

const logPromise = exports.logPromise = (promise, promiseName) => {
    const start = Date.now();
    promise
        .then((...res) => logSucceedTitle(`${promiseName} result:`, ...res, `in ${Date.now() - start}ms`))
        .catch(err => logDangerTitle(`${promiseName} error:`, err.message, `in ${Date.now() - start}ms`));
};

function log(style, ...tokens) {
    console.log(style(...tokens));
}

function logTitle(style, title, ...tokens) {
    console.log(style(title), ...tokens);
}